#include <stdio.h>

/*    :set tabstop=5    */

void	get ( fin, data, width, height, x, y, color, plane, colors, decimate_x, decimate_y, offset_x, nibs, trpin, vscale, dpi, swap )
	FILE      *fin;
	char	data[];
	int	width, height, x, y, decimate_x, decimate_y, swap;
	int	*color, colors[8];
	float *offset_x;
	int	*nibs;
	int	*trpin;
	float *vscale;
	float *dpi;
/* open and read an SIOSEIS plotfile, putting it in array data
*  SIOSEIS plot files start with 44 ASCII lines.  Line 1 is:
*  bytes 0-34 = SIOSEIS version number
*       35-58 = date the plot was created
*       68-71 = dots per inch.
*       72-75 = number of 32 bit words in every raster line
*       76-79 = number of raster lines
*
*  Paul Henkart,  Scripps Oceanography,  October 1994
*  (C) 1994, Regents of The University of California
*
* mod 28 Oct 04 - 0xFF the last line on color plots - must be a sioseis bug
* mod 17 Nov 04 - nlines needs to be static.
* mod 27 Feb 2009 - fix raster order for Intel (little endian)
*                   The bits in the byte (PutImage) are little endian too!
*/


{
	static int bytes_per_line_sio, nbytes, rem_y, first=1, nlines;
     int       endian, nwrds, i, j=0, k, kk;
	int		bits_per_line_x, bytes_per_line_x;
     char      sioseis_vers[36]= {" "},
               plot_date[32]= {" "}, line[81]= {" "}, gray[3]={" "};
	unsigned char	nib;
	char		sio[2000], sio1[2000], sio2[2000], junk[2000];
	int		huh, xbitno, xbyteno, bits_done, istat, nsio_done=0;
	long		address;
	int is_big_endian();


	if( first == 1 ) {
		first = 0;
		endian = is_big_endian();
		if( swap == 1 ) {
			if( endian < 0 ) endian = 0;
			else endian = -1;
		}
		istat = fseek( fin, 0, 0 );
		fread( &sioseis_vers, sizeof(char), 35, fin);
		fread( &plot_date, sizeof(char), 29, fin);
		fread( &gray, sizeof(char), 2, fin);
		fread( &line, sizeof(char), 14, fin);
		sscanf( line, "%d%d%d", nibs, &nwrds, &nlines);
          if( *nibs < 50 || *nibs > 10000 ) { printf
		   ("bad nibs.  Is the plotfile a SIOSEIS rasterfile?\n");
		   printf("nibs= %d, nwrds= %d, nlines= %d\n",*nibs,nwrds,nlines);
		}
		nbytes = nwrds * 4;
/*        now get the stuff in the last three lines of the sioseis plot header  */
		istat = fseek( fin, 3290, 0 );
		fscanf( fin, "%f", vscale );
		istat = fseek( fin, 3304, 0 );
		fscanf( fin, "%d", trpin );
		bytes_per_line_sio = get_nbytes_sio( *nibs, dpi );
/* if there are 3 planes it must be a color plot!  */
		address = bytes_per_line_sio * nlines * 2;
		istat = fseek( fin, address, 0 );
		*color = fread( &sio, sizeof(char), 1, fin);
		if( strcmp(gray,"g ")==0) *color = 2;
		*offset_x = *offset_x / 100. * (float)nlines;
		x = x + (long)*offset_x;
		rem_y = decimate_y - 1;
	}
	nsio_done = nsio_done + decimate_x - 1;
	bytes_per_line_x = width;
	if( *color == 0 ) bytes_per_line_x = width/8 + 1;
	if( plane != 0 ) address = address + (plane-1) *  bytes_per_line_sio * nlines;
	bits_per_line_x = bytes_per_line_x * 8;
	xbyteno = bytes_per_line_x-1;	/* byte number of the ximage */
	xbitno = 7;				/* bit number within the byte */
	for ( i=0; i<width; i++ ) {
		if( *color == 0 ) {
			address = 3520 + (x+nsio_done) * bytes_per_line_sio;
                	if( endian < 0 ) {
                        	huh = (( nsio_done / 8) * 8) + ( 7-(nsio_done % 8));
                        	address = 3520 + (x+huh) * bytes_per_line_sio;
                	}
			istat = fseek( fin, address, 0 );
			istat = fread( &sio, sizeof(char), nbytes, fin);
			if( istat == 0 ) for( j=0; j<nbytes; j++ ) 
				sio[j] = 0;
			j = xbyteno;	/* byte within ximage of first raster */
			bits_done = 0;
			for ( k=y/8; k<nbytes; k++ ) {
				for ( kk=0; kk<8; kk++ ) {
					if( bits_done % decimate_y == rem_y ) {
						nib = ( sio[k] >> (7-kk) ) & 1;
						data[j] = data[j] | (nib<<(7-xbitno));
						j = j +  bytes_per_line_x;
					}
					bits_done++;
				}
				if( bits_done+2 > height * decimate_y ) k = bytes_per_line_sio;
			}
			xbitno--;
			if( xbitno < 0 ) { xbyteno--; xbitno = 7; }
		} else {
			address = 3520 + ((x+nsio_done) * bytes_per_line_sio) * 3;
                	if( endian < 0 ) {
                        	huh = (( nsio_done / 8) * 8) + ( 7-(nsio_done % 8));
                        	address = 3520 + (x+huh) * bytes_per_line_sio * 3;
                	}
			istat = fseek( fin, address, 0 );
/*	 printf("sio, address=%d, istat=%d\n",address,istat); */
			istat = fread( &sio, sizeof(char), nbytes, fin);
			address = address + bytes_per_line_sio;
			istat = fseek( fin, address, 0 );
	 /*printf("sio1, address=%d, istat=%d\n",address,istat); 
			istat = fread(&sio1,sizeof(char),nbytes,fin);*/
			address = address + bytes_per_line_sio;
			istat = fseek( fin, address, 0 );
	/* printf("sio2, address=%d, istat=%d\n",address,istat); */
			istat = fread(&sio2,sizeof(char),nbytes,fin);
/*  the last raster line comes out wrong for some reason on after a move.
    Zero the critter.  Detect the last by trying to read the next one.  */
			address = address + bytes_per_line_sio;
			istat = fseek( fin, address, 0 );
			istat = fread(&junk,sizeof(char),nbytes,fin);
			if( nsio_done >= nlines-1 || istat == 0 ) for( j=0; j<bytes_per_line_sio; j++ )
					sio[j] = sio1[j] = sio2[j] = 0xFF;
               		j = xbyteno;   /* byte within ximage of first raster */
               		bits_done = 0;
               		for ( k=y/8; k<nbytes; k++ ) {
                    		for ( kk=0; kk<8; kk++ ) {
                         		if( bits_done % decimate_y == rem_y ) {
                              			data[j] = colors[(( sio[k] >> (7-kk) ) &1)
                               			| ((( sio1[k] >> (7-kk) ) & 1) << 1 )
                               			| ((( sio2[k] >> (7-kk) ) & 1) << 2 )];
                              			j = j +  bytes_per_line_x;
                         		}
                         		bits_done++;
                    		}
                    		if( bits_done+2 > height * decimate_y ) k = bytes_per_line_sio;
               		}
               		xbyteno--;
          	}
         	 nsio_done = nsio_done + decimate_x;
     	}
     	return;
}

