#define	debug	0
/*
int	get_header( fin, wanted_num, trpin, dpi, trace_header)
   Find and return the SEG-Y trace header from a SEG-Y trace header file.

Arguments:
fin - the file pointer of the SEG-Y trace header file (nothing but headers).
wanted_num - Find the header closest to this sequence number (word 1)
trpin - traces per inch 
trace_header - the return pointer of the trace header

Copyright (C)  The Regents of the University of California, April 2001
Written by Paul Henkart, Scripps Institution of Oceanography

mod Jan 02 - Use the last header in the file if the initial read is bad.
mod Jan 03 - Start from the beginning of the file every time - there's
             a bug in the backing up scheme!
           - Use dpi rather than nibs
mod Oct 07 - header file is now a segy file and thus need 3600 byte offset.
mod 29 Oct 10 - segy header is now in standard's byte order
*/

#include <stdio.h>
#include "segy_header.h"


int	get_header( fin, wanted_num, trpin, dpi, trace_header )
	FILE	*fin;
	long	wanted_num;
	int	trpin;
	float	dpi;
	struct    SEGY_TRACE_HEADER   *trace_header;

{
	int	is_big_endian();
	int	endian, istat;
	static long	last_num = 0, header_num, half;
	struct	SEGY_TRACE_HEADER a;

	last_num = 0;
	if( last_num == 0 ) {
		header_num = 0;
		half = dpi / trpin / 2 + 1 ;   /* watch out for truncation  */
	}
	istat = 1;
	endian = is_big_endian();
	while ( istat > 0 ) {
		if( wanted_num < last_num ) header_num = header_num -5;
		if( header_num < 0 ) header_num = 0;
/*        fseek always returns 0, whether a valid address or not   */
		istat = fseek( fin, header_num * HEADER_LENGTH + 3600, 0 );
     		istat = fread( &a, sizeof(char), HEADER_LENGTH, fin);
#if( debug )
		printf(" want %d, last %d, head %d\n",wanted_num,last_num,header_num);
		printf("istat= %d, shot %d trace %d rp %d trace %d\n",istat, 
			a.shot_num, a.shot_tr, a.rp_num, a.rp_tr);
#endif
		if( istat == 0 ) {
/*			istat = fseek( fin, (header_num-1) * HEADER_LENGTH + 3600, 0 );*/
			istat = fseek( fin, -1 * HEADER_LENGTH, 2 );
			istat = fread( &a, sizeof(char), HEADER_LENGTH, fin);
			istat = 0;
		}
		*trace_header = a;
		if( endian < 0 ) swap32( &trace_header->seq_num, 1 );
		if( trace_header->seq_num - half <= wanted_num &&
			wanted_num <= trace_header->seq_num + half ) return(istat);
		last_num = trace_header->seq_num;
		header_num++;
		if( header_num == 1 && wanted_num < last_num ) {
			fprintf( stderr, "Pick is before any data.\n");
			return(-1);
		}
	}
	
	return(istat);
}
