/* sioplt filename
*  View a SIOSEIS plotfile in an X window.
*  Use siopltrc as the resource file.
*  Paul Henkart,  Scripps Oceanography,  November 1994
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>


main(argc,argv)
	int	argc;
	char	*argv[];
{
	char		*progname = "sioplt";
	Display	*display;
	Visual	*visual;
	Window	window;
	GC		gc;
	XEvent    event;
	KeySym    key;
	XSizeHints     hints;
	Drawable	drawable;
	unsigned int depth;
	int	screen, format, bitmap_pad;
	XImage	*image;
	Colormap	cmap;
	XColor    xcolor[8];
	XTextProperty windowName, iconName;
	int		flags = { DoRed | DoGreen | DoBlue };
	int		src_x, src_y, dst_x, dst_y, bytes_per_line_x;
	int		forever=1, x=0, y=0, colors[8];
	char		*data, *d, *dd;
     FILE		*fin;
     int		nibs, nwrds, i,j=0, nbytes_per_line_sio, nlines, color=0;
	static char *cnames[] = { "Black","Red","Green","Yellow",
						"Blue", "Magenta", "Cyan", "White"};
	float	temp;

/*  The following are setable by the user via file siopltrc  */
	unsigned int width=500, height=500, reverse_video=0, plane=0;
	float	speed = 1.;

	if ( argc != 2 ) {
		fprintf (stderr, "usage: %s name\n", progname);
		exit(1);
	}
     if ((fin = fopen ( argv[1], "r" )) == (FILE *) NULL ) {
          printf ("Couldn't open file %s\n", argv[1]);
          exit(1);
     }

	display = XOpenDisplay("");
	if( display == 0 ) {
		fprintf( stderr, "Can not establish display connection.\n");
		printf("Did you   csh/tcsh:   setenv DISPLAY machine:0.0 ?\n");
		printf("          bash:   export DISPLAY machine:0.0 ?\n");
		exit(1);
	}
	screen = DefaultScreen ( display );
	depth = DefaultDepth ( display, screen );
	visual = DefaultVisual(display,screen);
	window = XCreateSimpleWindow (display, DefaultRootWindow(display),
		0, 0, width, height, 5, 0, 1);  /* fore/back doesn't matter */
	if( depth != 1 ) {
		cmap = DefaultColormap (display, screen); 
		for (i=0; i<8; i++ ) {
			if( !XParseColor (display, cmap, cnames[i],&xcolor[i])) {
				fprintf(stderr, "Color %s not found.\n",cnames[i]);
				exit(0);
			}
			printf(" rgb values = %d %d %d\n", xcolor[i].red,
				xcolor[i].green,xcolor[i].blue);
			if (!XAllocColor( display, cmap, &xcolor[i])) {
				fprintf(stderr, "Can't allocate color.\n");
				exit(0);
			}
			colors[i] = xcolor[i].pixel;
			printf("colors = %d\n",colors[i]);
		}
	}
	if (XStringListToTextProperty (&argv[1], 1, &windowName) == 0) {
		fprintf( stderr, "%s: structure allocation for windowName \
				failed.\n",progname);
		exit(-1);
	}
	if (XStringListToTextProperty (&progname, 1, &iconName) == 0) {
		fprintf( stderr, "%s: structure allocation for iconName \
				failed.\n",progname);
		exit(-1);
	}
	XSetWMProperties( display, window, &windowName, &iconName, argv,
		argc, NULL, NULL, NULL );
	get_sioplt_params( &width, &height, &reverse_video, &plane,
		 &speed );
	width = (width/8)*8; height = (height/8)*8;
	bitmap_pad = 8; 
/*   we don't know if it's color yet, so get enough mem in case it is */
	data = (char *)malloc ( width * height );
	if ( data == 0 ) {
		printf("Can not get memory, need %d\n", width * height );
		exit(0);
	}
	dd = data + width * height;
	for( d=data; d < dd; d++ ) *d = 0;	/* assume white background */
	get ( fin, data, width, height, x, y, &color, plane, colors );
	if( color == 1 && depth == 1 && plane == 0 ) {
		printf("Color can not be displayed on monochrome!\n");
		exit(0);
	}
	if( color == 1 ) bytes_per_line_x = width;	/*  color */
	else bytes_per_line_x = width / 8 +1;		/*  B & W */
	src_x = src_y = dst_x = dst_y = 0;
	XSelectInput (display, window,
		ButtonPressMask | KeyPressMask | ExposureMask );
	XMapRaised (display, window);
	gc = XCreateGC (display, window, 0, 0);
	if( color == 0 || plane != 0 ) {
		depth = 1;
		format = XYBitmap;
		if( reverse_video == 1 ) for( d=data; d < dd; d++ ) *d = ~*d;
	} else {
		depth = 8;
		format = ZPixmap;
	}
	image = XCreateImage ( display, visual, depth, format, 0,
			data, width, height, bitmap_pad, bytes_per_line_x  );
	
	XFlush( display );

	while (forever) {
		XNextEvent ( display, &event );
		switch (event.type) {
			case Expose:
				XPutImage ( display, window, gc, image, src_x,
					src_y, dst_x, dst_y, width, height );
				XFlush( display );
				break;
			case KeyPress:
				forever = 0;
				break;
			case ButtonPress:
				temp = ( (float)width/2. - (float)event.xbutton.x )
					* speed;
				x = x - (int)temp;
				if( x < 0 ) x = 0;
				temp = ( (float)height/2. - (float)event.xbutton.y )
					* speed;
				y = y + (int)temp;
				if( y < 0 ) y = 0;
				dd = data + bytes_per_line_x * height;
     			for( d=data; d < dd; d++ ) *d = 0;
				get (fin, data, width, height, x, y, &color, plane,
					 colors);
				if( reverse_video == 1 && depth == 1 )
					for( d=data; d < dd; d++ ) *d = ~*d;
				image = XCreateImage ( display, 
						DefaultVisual (display,screen), depth,
						format, 0, data, width, height,
						bitmap_pad, bytes_per_line_x  );
  	               XPutImage ( display, window, gc, image, src_x,
					src_y, dst_x, dst_y, width, height );
                    XFlush( display );
				break;
		}
	}
	XFreeGC ( display, gc );
	XDestroyWindow (display, window);
	XCloseDisplay (display);
	exit (0);
}
