#define	debug	0
/* :set tabstop=4 */
/* sioplt filename
*  View a SIOSEIS plotfile in an X window.
*  Use siopltrc as the resource file.
*  Paul Henkart,  Scripps Oceanography,  November 1994
*  mod spring 2001 to add picking
*  mod June 5, 2001 - Add XSetForeground and XSetBackground so that
*                     ZPixmap or B&W plots work right
*  mod Jan 2002 - Add -sets and -tp for arbitrary segy header values
*                 Add a bunch of others (indices)
*  mod Sep 2002 - Finish adding -vtp (started in feb?)
*               - Argh.  Redo the dot visual affirmation that I left ashore
*  mod Jan 2003 - Make sure -hf is given if any picking is done
*               - use dpi rather than rnibs since nibs is often the plotter
*                 model rather than the dot density!
*               - Time shouldn't use offset but does need origin.
*  mod Oct 2004 - Add siopltrc parameter curs_color.
*               - refine the error message about no header file.
*               - Save the last 20 picks
*               - Attempt to honor grayscale.
*  mod Oct 2005 - Change the way XYBitmap works so TrueColor (24 planes) works.
*               - Only way to quit is with the letter 'q'
*  mod Nov 2005 - Time is off after the first move.
*  mod Oct 2007 - change get_headers.c since header file is now a segy file.
*  mod 27 Feb 2009 - fix raster order for Intel (little endian)
*  mod 29 Oct 10 - segy header is now in standard's byte order
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/X.h>
#include <X11/cursorfont.h>
#include <stdio.h>
#include "segy_header.h"

#define	MAX_LIST 100	/* The maximum number of traces in twp  */
#define	MAX_PICKS 20	/* The maximum number of (x,y) picks to remember  */
#define	EMPTY	-1
#define	XOFFSET	1   /* .950001  sioseis routine ploted.f set xoffset.  Most nibs use 1.0.
                           We may have to put xoffset into the trace header  */


main(argc,argv)
	int	argc;
	char	*argv[];
{
	char		*progname = "sioplt";
	Display	*display;
	Visual	*visual;
	Window	window;
	GC		gc;
	XEvent    event;
	KeySym    key;
	XSizeHints     hints;
	Cursor	curs;
	unsigned int depth;
	int	screen, format, bitmap_pad;
	XImage	*image;
	Colormap	cmap;
	XColor    xcolor[8], exact[8];
	XTextProperty windowName, iconName;
	int		flags = { DoRed | DoGreen | DoBlue };
	int		src_x, src_y, dst_x, dst_y, bytes_per_line_x;
	int		forever=1, x=0, y=0, colors[8], ncolors=8;
	unsigned long	plane_masks[1], pixels_return[8];
	int		class, cells, planes, black, white;
	char		*data, *d, *dd;
     FILE		*fin, *fin_header=0, *fin_twp=0, *fin_wbt=0, *fin_ttp=0, *fin_tsets=0;
	FILE		*fin_sets=0, *fin_tp=0, *fin_vtp=0;
     int		pick=0, nibs, nwrds, i,j=0, nbytes_per_line_sio, nlines, color=0;
	int		trpin, istat, tr, last_tr=-1, m, n, xy=0;
	long		fno, lno, last_fno;
	int		list[MAX_LIST], list_index;
	int		shape=XC_crosshair;
	int		y_raster_offset=1;
	long		row=0, long_temp;
	int		x_origin=0, y_origin=0, delta_x=0, delta_y=0;
	int		last_picks_x[MAX_PICKS], last_picks_y[MAX_PICKS];
	int		endian, itemp;
     char      sioseis_vers[36]= {" "}, plot_date[32]= {" "}, gray[3]={" "};
/*	static char *cnames[] = { "Black","Red","Green","Yellow",
						"Blue", "Magenta", "Cyan", "White"};
*/
	static char *cnames[] = { "White","Cyan","Magenta","Blue",
						"Yellow", "Green", "Red", "Black"};
   static char *gnames[] = { "gray0", "gray13", "gray25", "gray37",
					   "gray50", "gray62", "gray75", "gray88" };
/*
  0   0   0         gray0
 33  33  33         gray13
 47  79  79         dark slate gray
 64  64  64         gray25
 94  94  94         gray37
105 105 105         dim gray
112 128 144         slate gray
119 136 153         light slate gray
127 127 127         gray50
158 158 158         gray62
190 190 190         gray
191 191 191         gray75
211 211 211         light grey
224 224 224         grey88
255 255 255         white
  0   0   0         black
c    The HP palette says:
c    black = 0, 0, 0
c    80% = 26, 26, 26
c    70% = 77, 77, 77
c    60% = 102, 102, 102
c    50% = 127, 127, 127
c    40% = 153, 153, 153
c    30% = 178, 178, 178
c    20% = 204, 204, 204
c    10% = 233, 233, 233
c    white = 255, 255, 255
c    sioseis says:
c    0    255 255 255         gray7     white
c    1    219 219 219         gray6
c    2    181 181 181         gray5
c    3    145 145 145         gray4
c    4    110 110 110         gray3
c    5     71  71  71         gray2
c    6     36  36  36         grey1
c    7      0   0   0         gray0     black
*/
		   float	last_time, time, time1=-99., time2=-99., vscale, temp, offset_x=0., offset_y=0.;
		   float	xfudge, yfudge, dpi, segy_value;
		   struct    SEGY_TRACE_HEADER   header;
		   struct	_XImage	ximage;
	   /*	char		str[10], *input_name = " ";
	   */
		   char		str[10];
		   char		*input_name="            ";
		   int		segy_index_type=' ';
		   int		segy_index = 0;
		   float     real_header[60];
		   long      long_header[60];
		   short     int_header[120];
		   void      move(), exit();


	   /*  The following are setable by the user via file siopltrc  */
		   unsigned int width=500, height=500, reverse_video=0, plane=0;
		   unsigned int decimate_x=1, decimate_y=1, bitswap=0; 
		   int		curs_color=0;
		   float	speed = 1.;

		   printf("sioplt version 29 October 2010\n");
		   if ( argc < 2 ) {
			   fprintf (stderr, "usage: %s name [options]\n", progname);
			   fprintf (stderr, "options:   x_offset\n");
			   fprintf (stderr, "           -if sioseis_plot_file\n");
			   fprintf (stderr, "           -hf segy_header_file\n");
			   fprintf (stderr, "           -twp output_twp_file  (trace and weight pairs)\n");
			   fprintf (stderr, "           -wbt output_wbt_file  (rp-number and time pair)\n");
			   fprintf (stderr, "           -ttp output_ttp_file  (trace-number and time pairs)\n");
			   fprintf (stderr, "           -vtp output_ttp_file  (velocity time pairs)\n");
			   fprintf (stderr, "           -rtp output_file real-segy-index time pairs\n");
			   fprintf (stderr, "           -ltp output_file long-segy-index time pairs\n");
			   fprintf (stderr, "           -itp output_file int-segy-index time pairs\n");
			   fprintf (stderr, "           -tsets output_tsets_file  (trace start/end time tuples)\n");
			   fprintf (stderr, "           -rsets output_file real-segy-index time pairs\n");
			   fprintf (stderr, "           -lsets output_file long-segy-index time pairs\n");
			   fprintf (stderr, "           -isets output_file int-segy-index time pairs\n");
			   fprintf (stderr, "           -xy\n");
			   exit(1);
		   }
		endian = is_big_endian();
		   n = 0;
		   m = argc-1;
		   while ( m > 0 ) {
			   /* this seems to be necessary for some queer reason  */
			   i = strcmp(argv[n+1],"-hf");
			   if( strncmp(argv[n+1],"-if",3) == 0 ) { 
	   /*    I give up on input_name. (XStringListToTextProperty (&input_name, 1, &windowName) == 0) 
		 keeps failing.
				   sscanf( argv[n+2], "%s",&input_name);
				   j = strlen(argv[n+2]);
				   strncpy(&input_name,argv[n+2],j);
	   */
				   if ((fin = fopen ( argv[n+2], "r" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-hf",3) == 0 ) { 
				   if ((fin_header = fopen ( argv[n+2], "r" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-twp",4) == 0 ) {
				   if ((fin_twp = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   pick = 1;
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-wbt",4) == 0 ) {
				   if ((fin_wbt = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   pick = 1;
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-ttp",4) == 0 ) {
				   if ((fin_ttp = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   pick = 1;
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-vtp",4) == 0 ) {
				   if ((fin_vtp = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   pick = 1;
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-tsets",6) == 0 ) {
				   if ((fin_tsets = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   pick = 1;
				   m -= 2;
				   n += 2;
			   } else if( strncmp(argv[n+1],"offset_x",8) == 0 ) {
				   sscanf( argv[n+2], "%f", &offset_x );
				   if( offset_x < 0 || offset_x >= 100. ) fprintf (stderr, 
					   "offset_x is a percentage of the plot file.\n");
				   m -= 2;
				   n += 2;
			   }else if( strncmp(argv[n+1],"-rsets",6) == 0 ) {
				   segy_index_type = 'r';
				   if ((fin_sets = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   sscanf( argv[n+3], "%d", &segy_index);
				   pick = 1;
				   m -= 3;
				   n += 3;
			   }else if( strncmp(argv[n+1],"-lsets",6) == 0 ) {
				   segy_index_type = 'l';
				   if ((fin_sets = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   sscanf( argv[n+3], "%d", &segy_index);
				   pick = 1;
				   m -= 3;
				   n += 3;
			   }else if( strncmp(argv[n+1],"-isets",6) == 0 ) {
				   segy_index_type = 'i';
				   if ((fin_sets = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   sscanf( argv[n+3], "%d", &segy_index);
				   pick = 1;
				   m -= 3;
				   n += 3;
			   }else if( strncmp(argv[n+1],"-rtp",4) == 0 ) {
				   segy_index_type = 'r';
				   if ((fin_tp = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   sscanf( argv[n+3], "%d", &segy_index);
				   pick = 1;
				   m -= 3;
				   n += 3;
			   }else if( strncmp(argv[n+1],"-ltp",4) == 0 ) {
				   segy_index_type = 'l';
				   if ((fin_tp = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   sscanf( argv[n+3], "%d", &segy_index);
				   pick = 1;
				   m -= 3;
				   n += 3;
			   }else if( strncmp(argv[n+1],"-itp",4) == 0 ) {
				   segy_index_type = 'i';
				   if ((fin_tp = fopen ( argv[n+2], "a" )) == (FILE *) NULL ) {
					   printf ("Couldn't open file %s\n", argv[n+2]);
					   exit(1);
				   }
				   sscanf( argv[n+3], "%d", &segy_index);
				   pick = 1;
				   m -= 3;
				   n += 3;
			   }else if( strncmp(argv[n+1],"-xy",3) == 0 ) {
				   xy = 1;
				   m -= 3;
				   n += 3;
			   } else {
				   fprintf( stderr, "Unknown option %s\n",argv[n+1]);
				   exit(1);
			   }
		   }
		   if( segy_index_type != ' ' ) {
			   if( segy_index <= 0 ||  segy_index > 120 ) {
				   printf("*** ERROR  ***  Bad SEG-Y index\n");
				   exit(1);
			   }
		   }
	   /*   set some variables that aren't set via static  */
		   for ( i=0; i<MAX_LIST; i++ ) list[i] = EMPTY;
		   last_fno = EMPTY;
		   list_index = 0;
		   for( i=0; i<MAX_PICKS; i++ ) {
			   last_picks_x[i] = 0;
			   last_picks_y[i] = 0;
		   }


		   display = XOpenDisplay("");
		   if( display == 0 ) {
				fprintf( stderr, "Can not establish display connection.\n");
				fprintf( stderr, "Use ssh -Y to login to the remote machine.\n");
/*				printf("Did you   csh/tcsh:   setenv DISPLAY machine:0.0 ?\n");
        		       	printf("          bash:   export DISPLAY machine:0.0 ?\n");
*/
				exit(1);
		   }
		   screen = DefaultScreen ( display );
		   depth = DefaultDepth ( display, screen );
		   visual = DefaultVisual(display,screen);
/*			class = visual.class;   */
			cells = XDisplayCells(display,screen);
			planes = XDisplayPlanes(display,screen);
			black = BlackPixel(display,screen);
			white = WhitePixel(display,screen);
#if( debug )
	printf("class= %d, cells= %d, planes= %d\n",class,cells,planes);
	printf("depth= %d, black= %X, white= %X\n",depth,black,white);
#endif
/*   visual.class
#define StaticGray       0
#define GrayScale        1
#define StaticColor      2
#define PseudoColor      3
#define TrueColor        4
#define DirectColor      5
*/
		   curs = XCreateFontCursor( display, shape );
		   get_sioplt_params( &bitswap, &width, &height, &reverse_video, &plane,
			    &speed, &decimate_x, &decimate_y, &curs_color );
	   /*   Watch out for reverse_video - it has opposite meaning from the
		   user's parameter because hardcopy plotters are opposite of screens! */
/*		   if( reverse_video == 0 ) reverse_video = 1;
		   else if( reverse_video == 1 ) reverse_video = 0;
		   else if( reverse_video == 2 ) reverse_video = 1;
*/
		   width = (width/8)*8-1; height = (height/8)*8;
		   window = XCreateSimpleWindow (display, DefaultRootWindow(display),
			   0, 0, width, height, 5, 0, 1);  /* fore/back doesn't matter on ZPixmap */
		   XDefineCursor( display, window, curs );
		   cmap = DefaultColormap (display, screen); 
		   fread( &sioseis_vers, sizeof(char), 35, fin);
		   fread( &plot_date, sizeof(char), 29, fin);
		   fread( &gray, sizeof(char), 2, fin);
		   if( strcmp(gray,"g ")==0) color = 2;
/*
		   if (XAllocColorCells (display, cmap, False, plane_masks, 0, pixels_return, ncolors) == 0 ) {
				fprintf (stderr, "basic: couldn't allocate read/write colors\n");
				exit(0);
		   }
*/
		   for (i=0; i<8; i++ ) {
			   if( color == 2 ) cnames[i] = gnames[i];
/*
			   if (!XParseColor( display, cmap, cnames[i], &exact[i])) {
				fprintf(stderr, "Color name %s is not in database.\n",cnames[i]);
				exit(0);
			   }
			   if (!XAllocColor( display, cmap, &exact[i])) {
				   fprintf(stderr, "Can't allocate color %s.\n",cnames[i]);
				   exit(0);
			   }
*/
			   if (!XAllocNamedColor( display, cmap, cnames[i], &xcolor[i], &exact[i])) { 
				   fprintf(stderr, "Can't allocate color %s.\n",cnames[i]);
				   exit(0);
			   }
			   colors[i] = exact[i].pixel;
		   }
		   XRecolorCursor (display, curs, &xcolor[curs_color], &xcolor[7]); 
		   if (XStringListToTextProperty (&input_name, 1, &windowName) == 0) {
			   fprintf( stderr, "%s: structure allocation for windowName \
					   failed.\n",progname);
			   exit(-1);
		   }
		   if (XStringListToTextProperty (&progname, 1, &iconName) == 0) {
			   fprintf( stderr, "%s: structure allocation for iconName \
					   failed.\n",progname);
			   exit(-1);
		   }
		   XSetWMProperties( display, window, &windowName, &iconName, argv,
			   argc, NULL, NULL, NULL );
		   bitmap_pad = 8; 
	   /*   we don't know if it's color yet, so get enough mem in case it is */
		   data = (char *)malloc ( width * height );
		   if ( data == 0 ) {
			   printf("Can not get memory, need %d\n", width * height );
			   exit(0);
		   }
		   dd = data + width * height;
	if( offset_x > 0. ) {
		x = offset_x;
		offset_x = 0.;
 	}
	if( color != 0 && depth == 1 && plane == 0 ) {
		printf("Color can not be displayed on monochrome!\n");
		exit(0);
	}
	if( color != 0 ) bytes_per_line_x = width;	/*  color */
	else bytes_per_line_x = width / 8 +1;		/*  B & W */
	src_x = src_y = dst_x = dst_y = 0;
	XSelectInput (display, window,
		ButtonPressMask | KeyPressMask | ExposureMask );
	XMapRaised (display, window);
	gc = XCreateGC (display, window, 0, 0);
	if( color == 0 || plane != 0 ) {
		depth = 1;
		format = XYBitmap;
		XSetForeground( display, gc, black );
		XSetBackground( display, gc, white );
/*		the foreground pixel value is applied to all pixels with a value 1 */
/* 		The background pixel value is applied to all pixels with a value 0 */
/*#define XYBitmap         0    /* depth 1, XYFormat */
/*#define XYPixmap         1    /* depth == drawable depth */
/*#define ZPixmap               2    /* depth == drawable depth */
	} else {
/*		background and foreground are ignored   */
/*		The pixel values are sent directly to the output pipeline.  */
/*		depth = 8;  */
		format = ZPixmap;
	}
	for( d=data; d < dd; d++ ) *d = 0;  /* XYBitmap, white */
	if( reverse_video == 1 ) for( d=data; d < dd; d++ ) *d = ~*d;
	get ( fin, data, width, height, x, y, &color, plane, colors,
		decimate_x, decimate_y, &offset_x, &nibs, &trpin, &vscale, &dpi, bitswap );
	if( dpi < 200 ) {
		xfudge = 9;
		yfudge = 1;
	} else {
		xfudge = 2;
		yfudge = 16;
	}
	ximage.bitmap_bit_order = MSBFirst;
	image = XCreateImage ( display, visual, depth, format, 0,
			data, width, height, bitmap_pad, bytes_per_line_x  );
	while (forever) {
		XNextEvent ( display, &event );
		switch (event.type) {
			case Expose:
				XPutImage ( display, window, gc, image, src_x,
					src_y, dst_x, dst_y, width, height);
				XFlush( display );
				break;
			case KeyPress:
				itemp = XLookupString( &event, str, 1, &key, 0 );
				if( str[0] != 'q' ) break;
				forever = 0;
				if( fin_header != 0 ) {
					if( fin_twp != 0 ) {
						sort( MAX_LIST-1, list );
						for ( j=0, n=0; j<MAX_LIST; j++ ) {
							if( list[j] != EMPTY ) {
								n++;
								fprintf(fin_twp,"%d 0 ",list[j]);
								if( n%10 == 9 ) 
									if( fin_twp != 0 ) fprintf(fin_twp,"\n       ");
							}
						}
						if( n > 0 ) {
							fprintf(fin_twp," end\n");
							printf(" end\n");
						}
					}
					if( fin_wbt != 0 ) fprintf(fin_wbt, "   %d  %f\n",last_fno,last_time);
					if( fin_ttp != 0 && last_tr != -1 ) {
						if( last_time != -99. ) fprintf(fin_ttp, "           %d  %f end\n",last_tr,last_time);
						else fprintf(fin_ttp, "           end\n");
						printf(" end\n");
					}
					if( fin_vtp != 0 ) {
						fprintf(fin_vtp, "           end\n");
						printf(" end\n");
					}
					if( fin_tp != 0 && last_tr != -1 ) {
						if( last_time != -99. ) fprintf(fin_tp, "%f  %f\n",segy_value,last_time);
					}
					if( fin_tsets != 0 && last_tr != -1 ) {
						fprintf(fin_tsets," %d %f %f end\n",last_tr,time1,time2);
						printf(" end\n");
					}
					if( fin_sets != 0 ) {
						fprintf(fin_sets," %f %f %f\n",segy_value,time1,time2);
						printf(" %f %f %f\n",segy_value,time1,time2);
					}
				}
				break;
			case ButtonPress:
#if( debug )
	printf("event.button: button= %d, x= %d, y= %d\n",event.xbutton.button,event.xbutton.x,event.xbutton.y);
#endif
				switch( event.xbutton.button ) {
					case 1:
						temp = ( (float)width/2. - (float)event.xbutton.x ) * speed;
						delta_x = -(int)temp;
						x_origin += delta_x;
						if( x_origin < 0 ) {
							delta_x -= x_origin;
							x_origin = 0;
						}
						for( i=0; i<MAX_PICKS; i++ ) {
							if( last_picks_x[i] != 0 ) {
								last_picks_x[i] += delta_x;
								if( last_picks_x[i] > width || last_picks_x[i] < 0 ) {
									last_picks_x[i] = 0;
									last_picks_y[i] = 0;
								}
							}
						}
						row = row + delta_x;
						x = x + delta_x * decimate_x;  /* x in the original raster file */
						if( row < 0 ) {
							row = x;
							x = 0;
						}
						temp = ( (float)height/2. - (float)event.xbutton.y ) * speed;
						delta_y = -(int)temp;
						y_origin += delta_y;   /* this is hard because +y is DOWN the screen  */
						if( y_origin < -height/2 ) {
							delta_y -= y_origin;
							y_origin = 0;
						}
						for( i=0; i<MAX_PICKS; i++ ) {
							if( last_picks_y[i] != 0 ) last_picks_y[i] += delta_y;
						}
/*						y_raster_offset += temp; */
						y_raster_offset -= delta_y;
						y = y - delta_y * decimate_y;
						if( y < 0 ) {
							y_raster_offset -= y;
							y = 0;
						}
						dd = data + bytes_per_line_x * height;
     					for( d=data; d < dd; d++ ) *d = 0;  /* XYBitmap, */
						get (fin, data, width, height, x, y, &color, plane,
					 		colors, decimate_x, decimate_y, &offset_x, &nibs, &trpin, &dpi, bitswap );
						if( reverse_video == 1 && depth == 1 )
							for( d=data; d < dd; d++ ) *d = ~*d;
						for( i=0; i<MAX_PICKS; i++ ) {
							if( last_picks_x[i] > 0 && last_picks_y[i] > 0 ) {
								dd = data + last_picks_x[i]/8;
								for( j=0; j<last_picks_y[i]-5; j++ ) dd = dd + width/8+1;
								for( j=-5; j<5; j++ ) {
									*dd = ~*dd;
									dd = dd + width/8+1;
								}
							}
						}
						image = XCreateImage ( display, visual, depth,
								format, 0, data, width, height,
								bitmap_pad, bytes_per_line_x  );
  	               		XPutImage ( display, window, gc, image, src_x,
							src_y, dst_x, dst_y, width, height);
                    		XFlush( display );
						yfudge = 14; 
						break;
					case 2:    /* the oops button  */
						dd = data + last_picks_x[0]/8;
						for( i=0; i<last_picks_y[0]-5; i++ ) dd = dd + width/8+1;
						for( i=-5; i<5; i++ ) {
							*dd = ~*dd;
							dd = dd + width/8+1;
						}
						image = XCreateImage ( display, 
								DefaultVisual (display,screen), depth,
								format, 0, data, width, height,
								bitmap_pad, bytes_per_line_x );
  	               		XPutImage ( display, window, gc, image, src_x,
							src_y, dst_x, dst_y, width, height);
                    		XFlush( display );
						if( fin_header == 0 && pick == 0 ) {
                                  fprintf (stderr,"Button 2 not honored without picking.\n");
                                  break;
						}
						if( fin_header == 0 && pick != 0 ) {
							fprintf (stderr,"***  ERROR  ***  SIOPLT -hf option must be given.\n");
							exit(1);
						}
						long_temp = row+((float)width-(float)event.xbutton.x+xfudge);
						istat = get_header( fin_header, long_temp,
							 trpin, dpi, &header );
						if( header.rp_tr == 0 ) {
							fno = header.shot_num;
							tr = header.shot_tr;
						}else{
							fno = header.rp_num;
							tr = header.rp_tr;
						}
						if( endian < 0 ) {
							swap32( &fno,1);
							swap32( &tr,1);
						}
						if( fin_twp != 0 && fno == last_fno ) {
							for( j=0; j<i; j++ ) {
								if( list[j] == tr ) {
									list[j] = EMPTY;
									break;
								}
							}
							printf("%d XX ",tr);
							fflush(stdout);
						}
						if( fin_wbt != 0 ) {
							printf("%d IGNORED\n",last_fno);
							last_fno = EMPTY;
						}
						if( fin_ttp != 0 || fin_vtp != 0 ) {
							printf("%d XX\n",last_tr);
							fflush(stdout);
							last_time = -99.;
							last_tr = -1.;
						}
						if( fin_tp != 0 ) {
							printf("%f XX\n",segy_value);
							fflush(stdout);
							last_time = -99.;
							last_tr = -1.;
						}
						if( fin_tsets != 0 || fin_sets != 0 ) {
							if( time2 == -99. ) {
								time1 = -99.;
								last_tr = -1.;
							}
							else time2 = -99.;
							printf(" XXX");
							fflush(stdout);
						}
						break;
					case 3:    /* the pick button  */
						/*  most cursors may not be centered.  The default (arrow)
						  needs to have 10 add to the x coordinate  */
						if( xy ) printf("x= %d, y= %d\n",event.xbutton.x, event.xbutton.y );
						for( i=MAX_PICKS-1; i>0; i-- ) {
							last_picks_x[i] = last_picks_x[i-1];
							last_picks_y[i] = last_picks_y[i-1];
						}
						last_picks_x[0] = event.xbutton.x;
						dd = data + (event.xbutton.x)/8;
						last_picks_y[0] = event.xbutton.y;
/*						dd = dd + ((event.xbutton.y+4 ) * width )/8;*/
#if( debug )
/*   row is the sioseis raster line (row) count for finding which trace it is  */
	printf("%d %d %d\n",row+(width-event.xbutton.x),trpin,nibs);
	printf("data= %d, dd= %d, event.xbutton.y= %d\n",data,dd,event.xbutton.y);
#endif
						for( i=0; i<event.xbutton.y-5; i++ ){
							dd = dd + width/8+1;
						}
						for( i=-5; i<5; i++ ) {
			/*   I think the following fails because on int arithmetic
            perhaps making things long will help, but I need to move on
			/*				dd = data + event.xbutton.x/8 + 
								((event.xbutton.y + i) * width )/8+i ; */
							*dd = ~*dd;
							dd = dd + width/8+1;
						}
						image = XCreateImage ( display, 
								DefaultVisual (display,screen), depth,
								format, 0, data, width, height,
								bitmap_pad, bytes_per_line_x );
  	               		XPutImage ( display, window, gc, image, src_x,
							src_y, dst_x, dst_y, width, height);
                    		XFlush( display );
						if( fin_header == 0 && pick == 0 ) {
                                  fprintf (stderr,"Button 3 not honored without picking.\n");
                                  break;
						}
						if( fin_header == 0 ) {
							fprintf (stderr,"***  ERROR  ***  SIOPLT -hf option must be given.\n");
							exit(1);
						}
						long_temp = (float)row+((float)width-(float)event.xbutton.x+xfudge);
						istat = get_header( fin_header, long_temp,
							 trpin, dpi, &header );
						if( istat < 0 ) {
							if( fin_ttp != 0 || fin_tp != 0 || fin_vtp != 0 ) {
								last_time = -99.;
								last_tr = -1.;
							}
							if( fin_tsets != 0 || fin_sets != 0 ) {
								if( time2 == -99. ) {
									time1 = -99.;
									last_tr = -1.;
								}
								else time2 = -99.;
							}
							break;
						}
						if( header.rp_tr == 0 ) {
							fno = header.shot_num;
							tr = header.shot_tr;
						}else{
							fno = header.rp_num;
							tr = header.rp_tr;
						}
						if( endian < 0 ) {
							swap32( &fno,1);
							swap32( &tr,1);
						}
						if( fin_tp != 0 || fin_sets != 0 ) {
							move(&header,&real_header,240);
							move(&header,&long_header,240);
							move(&header,&int_header,240);
						}
						time = ( (float)event.xbutton.y + yfudge - y_origin - (XOFFSET * dpi))
							 / dpi / vscale + header.delay;
#if( debug )
	printf("time= %f, dpi= %f, vscale= %f, del= %f\n",time,dpi,vscale,header.delay);
	printf("y_raster_offset= %d, event.xbutton.y= %d, XOFFSET= %f\n",y_raster_offset,event.xbutton.y,XOFFSET);
#endif
						/* write the results  */
						if( fin_twp != 0 && last_fno == EMPTY ) {
							printf("    fno %d twp ",fno);
							fflush(stdout);
							fprintf( fin_twp,"    fno %d twp ",fno);
						}
						if( fin_ttp != 0 && last_fno == EMPTY ) {
							printf("    fno %d lno %d ttp\n",fno,fno);
							fflush(stdout);
							fprintf( fin_ttp,"    fno %d lno %d ttp\n",fno,fno);
						}
						if( fin_vtp != 0 && last_fno == EMPTY ) {
						/* Remember the first trace raster is at 300  */
							istat = get_header( fin_header, 300, trpin, dpi, &header );
							fno = header.rp_num;
							istat = get_header( fin_header, 999999, trpin, dpi, &header );
							lno = header.rp_num;
							/* now get the hedaer for this point again!   */
							long_temp = (float)row+((float)width-(float)event.xbutton.x+xfudge);
							istat = get_header( fin_header, long_temp,
								trpin, dpi, &header );
/*							printf("    fno %d lno %d vtp \n           %d  %f\n",fno,lno,header.wvel,time); */
							printf("    fno %d vtp \n           %d  %f\n",(fno+lno+1)/2,header.wvel,time);
							fflush(stdout);
							fprintf( fin_vtp,"    fno %d lno %d vtp \n           %d  %f\n",fno,fno,header.wvel,time);
						}
						if( fin_tsets != 0 && last_fno == EMPTY ) {
							printf("    fno %d tsets ",fno);
							fflush(stdout);
							fprintf( fin_tsets,"    fno %d tsets ",fno);
						}
						/* is this different from the last pick and not the first?  */
						if( fno != last_fno && last_fno != EMPTY ) {
							if( fin_twp != 0 ) {
								sort( MAX_LIST-1, list );
								/* there maybe erasures (-1) in list */
								for ( j=0, n=0; j<MAX_LIST; j++ ) {
									if( list[j] != EMPTY ) {
										fprintf(fin_twp,"%d 0 ",list[j]);
										n++;
										if( n%10 == 9 ) fprintf(fin_twp,"\n       ");
									}
									list[j] = EMPTY;
								}
								list_index = 0;
								printf(" end\n");
								fprintf(fin_twp," end\n");
								printf("    fno %d twp ",fno);
								fprintf(fin_twp,"    fno %d twp ",fno);
							}
							/* write the previous wbt to the output file  */
							if( fin_wbt != 0 ) fprintf(fin_wbt, "   %d  %f\n",last_fno,last_time);
							if( fin_ttp != 0 && last_tr > 0 && last_time != -99. )
								fprintf(fin_ttp, "           %d  %f\n",last_tr,last_time);
							if( fin_tsets != 0 ) {
								printf(" end\n");
								fprintf(fin_tsets," end\n");
								printf("    fno %d tsets ",fno);
								fprintf(fin_tsets,"    fno %d tsets ",fno);
							}
						} else {
							if( fin_wbt != 0 && fno == 0 ) {
								printf("wbt (water bottom time) picking requires the data to have the rp number set.\n");
								printf("Use SIOSEIS process GEOM to set the rp numbers.\n");
							}
						}
						if( fin_ttp != 0 && last_tr > 0 && last_time != -99. ) {
							fprintf(fin_ttp, "           %d  %f\n",last_tr,last_time);
/*							printf("           %d  %f\n",last_tr,last_time);*/
						}
						if( fin_vtp != 0 && last_tr > 0 && last_time != -99. ) {
							fprintf(fin_vtp, "           %d  %f\n",header.wvel,last_time);
							printf("           %d  %f\n",header.wvel,time);
						}
						if( fin_tp != 0 && last_tr > 0 && last_time != -99. ) {
							fprintf(fin_tp, "%f  %f\n",segy_value,last_time);
/*							printf("%f  %f\n",segy_value,last_time); */
						}
						if( fin_tsets != 0 || fin_sets !=0 ) {
							/* Let the second time be ok even if it's from
							a different trace.  Lining up on the same trace
							can be hard.
							But do force the second time to be AFTER the first.  */
							if( time2 != -99. ) {
								if( time1 >= time2 ) {
									printf("ERROR: Time must increase.\n");
								} else { 
									if( fin_tsets != 0 )
										fprintf(fin_tsets," %d %f %f\n",last_tr,time1,time2);
									else
										fprintf(fin_sets," %f %f %f\n",segy_value,time1,time2);
									fflush(stdout);
									time1 = time2 = -99.;
								}
							}
							if( time1 == -99. ) {
								if( fin_tsets != 0 ) printf(" %d %f",tr,time);
								else printf(" %f %f",segy_value,time);
								time1 = time ;
								last_tr = tr;
							} else {
								printf(" %f \n",time);
								time2 = time;
							}
							fflush(stdout);
						}
						/*  finished with the last, do this one!  */
						if( fin_twp != 0 ) {
							list[list_index] = tr;
							list_index++;
							printf("%d 0 ",tr);
							fflush(stdout);
						}
						if( fin_wbt != 0 ) printf(" %d  %f\n",fno,time);
/*						if( fin_ttp != 0 ) {
							printf(" %d  %f\n",tr,time);
							fflush(stdout);
						}
*/
/*			printf("time= %f, time1= %f, time2= %f, last_time= %f\n",time,time1,time2,last_time);*/
						last_fno = fno;
						last_time = time;
						last_tr = tr;
						if( fin_tp != 0 || fin_sets != 0 ) {
							switch ( segy_index_type ) {
							case 'r':
								segy_value = real_header[segy_index-1];
								break;
							case 'l':
								segy_value = (float)long_header[segy_index-1];
								break;
							case 'i':
								segy_value = (float)int_header[segy_index-1];
								break;
							}
						}
						if( fin_ttp != 0 && last_tr > 0 && last_time != -99. )
							printf("           %d  %f\n",last_tr,last_time);
						if( fin_tp != 0 && last_tr > 0 && last_time != -99. ) 
							printf("%f  %f\n",segy_value,last_time); 
						break;
				}
		}
	}
	XFreeGC ( display, gc );
	XDestroyWindow (display, window);
	XCloseDisplay (display);
	exit (0);
}

void move(from,to,count)
char *from;
char *to;
int count;
{
  while(count--) *to++ = *from++;
}
