/* sioplt filename
*  View a SIOSEIS plotfile in an X window.
*  Use siopltrc as the resource file.
*  Paul Henkart,  Scripps Oceanography,  November 1994
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
/*#include <X11/Xos.h>*/
#include <stdio.h>

#define	USE_DEFAULT_COLORMAP	1
#define	USE_STANDARD_COLORMAP	0


main(argc,argv)
	int	argc;
	char	*argv[];
{
	char		*progname = "sioplt";
	Display	*display;
	Visual	*visual;
	XVisualInfo	visual_info, vTemplate, *visualList;
	XVisualInfo	*vlist, vinfo_template, *v;
	Window	window;
	GC		gc;
	XEvent    event;
	KeySym    key;
	XSizeHints     hints;
	unsigned int depth;
	int	screen, format, bitmap_pad, visualsMatched;
	XImage	*image;
	Colormap	cmap;
	XColor    xcolor[16];
	XTextProperty windowName, iconName;
	XStandardColormap *best_map_info;
	XColor	*exact_defs;
	XSetWindowAttributes	attrib;
	unsigned long attribmask;
	int		i, j, k, l, ncells;
	int		num_vis, count, strategy = USE_STANDARD_COLORMAP;
	int		flags = { DoRed | DoGreen | DoBlue };
	int		src_x, src_y, dst_x, dst_y, bytes_per_line_x;
	int		forever=1, x=0, y=0, colors[16];
	char		*data, *d, *dd;
     FILE		*fin;
     int		nibs, nwrds, ii=5,jj=0, nbytes_per_line_sio, nlines, color=0;
static	char	*visual_class[] = {
		"StaticGray","GrayScale","StaticColor",
		"PseudoColor","TrueColor","DirectColor"};
/*   X11 says gray 0 to gray100 exist, but the sun display craps out with:
"gray100","gray85","gray71","gray57",
"gray42","gray29","gray14","gray0"};
sioplt siofil
X Error of failed request:  BadColor (invalid Colormap parameter)
  Major opcode of failed request:  92 (X_LookupColor)
  Resource id in failed request:  0x0
  Serial number of failed request:  31
  Current serial number in output stream:  31
*/

	static char *cnames[] = { "Black","Red","Green","Yellow",
						"Blue", "Magenta", "Cyan", "White",
						"black","dark slate gray","dim gray","slate gray",
						"light slate gray","gray","light gray","white"};
	/*static char *gnames[] = { "gray0","gray14","gray29","gray42",
                              "gray57","gray71","gray85","gray100"};
*/
	float	temp, offset_x=0.;

/*  The following are setable by the user via file siopltrc  */
	unsigned int width=500, height=500, reverse_video=2, plane=0;
	unsigned int decimate_x=1, decimate_y=1;
	float	speed = 1.;

	if ( argc < 2 || argc > 3 ) {
		fprintf (stderr, "usage: %s name [offset_x]\n", progname);
		exit(1);
	}
     if ((fin = fopen ( argv[1], "r" )) == (FILE *) NULL ) {
          printf ("Couldn't open file %s\n", argv[1]);
          exit(1);
     }
	if( argc == 3 ) {
		sscanf( argv[2], "%f", &offset_x );
		if( offset_x < 0 || offset_x >= 100. ) fprintf (stderr, 
			"offset_x is a percentage of the plot file.\n");
	}

	display = XOpenDisplay("");
	if( display == 0 ) {
		fprintf( stderr, "Can not establish display connection.\n");
		printf("Did you   setenv DISPLAY machine:0.0 ?\n");
		exit(1);
	}
	screen = DefaultScreen ( display );
	depth = DefaultDepth ( display, screen );
	visual = DefaultVisual(display,screen);
	if (XGetRGBColormaps(display, RootWindow(display,screen),
			&best_map_info, &count, XA_RGB_BEST_MAP) == 0 ) {
		printf("RGB_BEST_MAP colormap property not set.\n");
		get_colors();
		strategy = USE_DEFAULT_COLORMAP;
		}
	else if (best_map_info->colormap) {
		if (best_map_info->red_max == 0 ) {
			printf("RGB_BEST_MAP colormap property is set, but missing data.\n");
			strategy = USE_DEFAULT_COLORMAP;
			}
		else {
			printf("std colormap ID: %d\n",best_map_info->colormap);
			attrib.colormap = best_map_info->colormap;
		}
	} else if (best_map_info->visualid == 0 ) {
		printf("std colormap property is set, but missing data.\n");
		strategy = USE_DEFAULT_COLORMAP;
		}
	else {
		vlist = XGetVisualInfo(display, VisualNoMask, 
			&vinfo_template, &num_vis);
		for (v=vlist; v<vlist+num_vis; v++ ) {
			if (v->visualid == best_map_info->visualid) {
				visual = v->visual;
				break;
				}
			}
		best_map_info->colormap = XCreateColormap(display,
			RootWindow(display,screen), visual, AllocAll);
		if (best_map_info->colormap == DefaultColormap(display,screen)){
			printf("hardware colormap is immutable.\n");
			printf("Cannot create new colormap.\n");
			}
		attrib.colormap = best_map_info->colormap;
		ncells = best_map_info->base_pixel +
			((best_map_info->red_max + 1) *
			(best_map_info->green_max + 1) *
			(best_map_info->blue_max + 1));
		exact_defs = (XColor *) calloc(sizeof(XColor), ncells);
		l = best_map_info->base_pixel;
		for ( i = 0; i < best_map_info->blue_max; i++) {
			for ( j = 0; j < best_map_info->blue_max; j++ ) {
				for ( k= 0; k < best_map_info->blue_max; k++) {
					exact_defs[l].red = 0xFFFF * k /
						best_map_info->red_max;
					exact_defs[l].green = 0xFFFF * j /
						best_map_info->green_max;
					exact_defs[l].blue = 0xFFFF * i /
						best_map_info->blue_max;
					l++;
					}
				}
			}
		XstoreColors (display,best_map_info->colormap, exact_defs, ncells);
		XSetRGBColormaps(display, RootWindow(display,screen),
			&best_map_info, count, XA_RGB_BEST_MAP);
		}
	if (strategy == USE_STANDARD_COLORMAP) {
		background_pixel = best_map_info->base.pixel + 
			(best_map_info->red_max * best_map_info->red_mult) +
			(best_map_info->green_max * best_map_info->green_mult) +
			(best_map_info->blue_max * best_map_info->blue_mult);
		attribmask = CSBackPixel | CWColormap;
		foreground_pixel = (best_map_info->green_max *
			 best_map_info->green_mult / 2) +
			 best_map_info->base_pixel;
		border_pixel = ( best_map_info->blue_max *
			 best_map_info->blue_mult / 2 ) +
			 best_map_info->base_pixel;
		}
	win = XCreateWindow(display, RootWindow(display,screen), x,
		y, width, height, borderwidth, DefaultDepth(display,screen),
		InputOutput, visual, attribmask, &attrib);

				


	get_sioplt_params( &width, &height, &reverse_video, &plane,
		 &speed, &decimate_x, &decimate_y );
/*   Watch out for reverse_video - it has opposite meaning from the
     user's parameter because hardcopy plotters are opposite of screens! */
     if( reverse_video == 0 ) reverse_video = 1;
     else if( reverse_video == 1 ) reverse_video = 0;
     else if( reverse_video == 2 ) reverse_video = 1;
	width = (width/8)*8; height = (height/8)*8;
	window = XCreateSimpleWindow (display, DefaultRootWindow(display),
		0, 0, width, height, 5, 0, 1);  /* fore/back doesn't matter */
	vTemplate.screen = screen;
	vTemplate.depth = depth;
	printf("depth=%d\n",depth);
	printf("Mask: %x %x\n",VisualScreenMask,VisualDepthMask);
	visualList = XGetVisualInfo( display, VisualScreenMask 
		/* | VisualDepthMask*/, &vTemplate, &visualsMatched );
	printf("visualsMatched= %d\n",visualsMatched);
	for( ii=0;ii<6;ii++ ){
		jj=XMatchVisualInfo(display,screen,depth,ii,&visual_info);
		printf("jj=%d, %s, cmap_size %d\n",jj,visual_class[ii],visual_info.colormap_size);
	}
	if( depth != 1 ) {
		cmap = DefaultColormap (display, screen); 
		for (i=0; i<16; i++ ) {
/*   for gray scale, use gnames rather than cnames  */

			if( !XParseColor (display, cmap, cnames[i],&xcolor[i])) {
				fprintf(stderr, "Color %s not found.\n",cnames[i]);
				exit(0);
			}
	printf("i=%d, cname=%s\n",i,cnames[i]);
			if (!XAllocColor( display, cmap, &xcolor[i])) {
				fprintf(stderr, "Can't allocate color.\n");
				exit(0);
			}
			colors[i] = xcolor[i].pixel;
/*	printf("%s %d\n",cnames[i],colors[i]);*/
		}
	}
	if (XStringListToTextProperty (&argv[1], 1, &windowName) == 0) {
		fprintf( stderr, "%s: structure allocation for windowName \
				failed.\n",progname);
		exit(-1);
	}
	if (XStringListToTextProperty (&progname, 1, &iconName) == 0) {
		fprintf( stderr, "%s: structure allocation for iconName \
				failed.\n",progname);
		exit(-1);
	}
	XSetWMProperties( display, window, &windowName, &iconName, argv,
		argc, NULL, NULL, NULL );
	bitmap_pad = 8; 
/*   we don't know if it's color yet, so get enough mem in case it is */
	data = (char *)malloc ( width * height );
	if ( data == 0 ) {
		printf("Can not get memory, need %d\n", width * height );
		exit(0);
	}
	dd = data + width * height;
	for( d=data; d < dd; d++ ) *d = 0;	/* assume white background */
	get ( fin, data, width, height, x, y, &color, plane, colors,
		decimate_x, decimate_y, &offset_x );
	if( offset_x > 0. ) {
		x = offset_x;
		offset_x = 0.;
 	}
	if( color == 1 && depth == 1 && plane == 0 ) {
		printf("Color can not be displayed on monochrome!\n");
		exit(0);
	}
	if( color == 1 ) bytes_per_line_x = width;	/*  color */
	else bytes_per_line_x = width / 8 +1;		/*  B & W */
	src_x = src_y = dst_x = dst_y = 0;
	XSelectInput (display, window,
		ButtonPressMask | KeyPressMask | ExposureMask );
	XMapRaised (display, window);
	gc = XCreateGC (display, window, 0, 0);
	if( color == 0 || plane != 0 ) {
		depth = 1;
		format = XYBitmap;
		if( reverse_video == 1 ) for( d=data; d < dd; d++ ) *d = ~*d;
	} else {
		depth = 8;
		format = ZPixmap;
	}
	image = XCreateImage ( display, visual, depth, format, 0,
			data, width, height, bitmap_pad, bytes_per_line_x  );
	
	XFlush( display );

	while (forever) {
		XNextEvent ( display, &event );
		switch (event.type) {
			case Expose:
				XPutImage ( display, window, gc, image, src_x,
					src_y, dst_x, dst_y, width, height);
				XFlush( display );
				break;
			case KeyPress:
				forever = 0;
				break;
			case ButtonPress:
				temp = ( (float)width/2. - (float)event.xbutton.x )
					* speed;
				x = x - (int)temp * decimate_x;
				if( x < 0 ) x = 0;
				temp = ( (float)height/2. - (float)event.xbutton.y )
					* speed;
				y = y + (int)temp * decimate_y;
				if( y < 0 ) y = 0;
				dd = data + bytes_per_line_x * height;
     			for( d=data; d < dd; d++ ) *d = 0;
				get (fin, data, width, height, x, y, &color, plane,
					 colors, decimate_x, decimate_y, &offset_x);
				if( reverse_video == 1 && depth == 1 )
					for( d=data; d < dd; d++ ) *d = ~*d;
				image = XCreateImage ( display, 
						DefaultVisual (display,screen), depth,
						format, 0, data, width, height,
						bitmap_pad, bytes_per_line_x  );
  	               XPutImage ( display, window, gc, image, src_x,
					src_y, dst_x, dst_y, width, height);
                    XFlush( display );
				break;
		}
	}
	XFreeGC ( display, gc );
	XDestroyWindow (display, window);
	XCloseDisplay (display);
	exit (0);
}
